<?php

/**
 * Settings Controller for Pressbackup.
 *
 * This Class provide a interface to display and manage Plugin settings
 * Also Manage the configurattion wizard
 *
 * Licensed under The GPL v2 License
 * Redistributions of files must retain the above copyright notice.
 *
 * @link			http://pressbackup.com
 * @package			controlers
 * @subpackage		controlers.settings
 * @since			0.1
 * @license			GPL v2 License
 */
 
class PressbackSettings {

	/**
	 * Redirect the user to de correct page
	 * If the user has not configured the plugin
	 * this function redirect directly to the config init page
	 *
	 * Note: this function is automaticaly called
	 * by pressing on settings menu link
	 */
	function index() {
		global $pressbackup;
		$preferences = get_option('pressbackup_preferences');

		if ($preferences['pressbackup']['configured']) {
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_credentials'));
		}
		$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_init'));
	}

	/**
	 * Start The configuration wizard
	 * Check all that all nedded libs are intalled
	 * Also check for perms on tmp folders
	 */
	function config_init() {

		global $pressbackup;
		$pressbackup->View->layout('principal');
		$preferences = get_option('pressbackup_preferences');

		//misc
		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		$error = array();
		$disable_backup_files = false;

		//check for
		if (strpos($_SERVER['SERVER_SOFTWARE'], 'iis') !== false) {
			$error['iss'] = __('Sorry your web Server (Windows IIS) is not complatible with Pressbackup. You are welcome to use Pressbackup, but we cannot provide any support or guarantees for your system','pressbackup');
		}

		//check for
		if (ini_get('safe_mode')) {
			$error['safe_mode'] = __('Safe mode is enabled. You would need to contact your hosting provider and ask them if it\'s  necessary for you to have safe_mode enabled. Most hosts should be able  to disable it for you, or give you a more specific answer','pressbackup');
			$disable_backup_files = true;
		}

		//check for Zip Creation
		$bin = $misc->checkShell('zip');
		if (!$bin) {
			$preferences['pressbackup']['compatibility']['zip'] = 10;
			update_option('pressbackup_preferences', $preferences);
		}
		if (!$bin && !class_exists('ZipArchive')) {
			$error['zip'] = __('You probably don\'t have the php-zip extension installed. You would need to contact your hosting provider and ask them to install it.','pressbackup');
			$disable_backup_files = true;
		}

		// Check for CURL
		if (!extension_loaded('curl')) {
			$error['curl'] = __('You probably don\'t have the php-curl extension installed.  You would need to contact your hosting provider and ask them to install it.','pressbackup');
			$disable_backup_files = true;
		}

		//check for
		if (!class_exists('SimpleXMLElement')) {
			$error['xml'] = __('You probably don\'t have the SimpleXML extension installed.  You would need to contact your hosting provider and ask them to install it.','pressbackup');
			$disable_backup_files = true;
		}

		//tmp dir
		if (!file_exists($pressbackup->Path->Dir['PBKTMP']) && !mkdir($pressbackup->Path->Dir['PBKTMP'])) {
			$error['tmpdir'] = sprintf(__('Could not create the FileStore directory "%1$s". Please check the effective permissions.','pressbackup'), $pressbackup->Path->Dir['PBKTMP']);
			$disable_backup_files = true;
		} else {
			@chmod($pressbackup->Path->Dir['PBKTMP'], 0777);
		}

		//log dir
		if (!file_exists($pressbackup->Path->Dir['LOGTMP']) && !mkdir($pressbackup->Path->Dir['LOGTMP'])) {
			$error['logdir'] = sprintf(__('Could not create the FileStore directory "%1$s". Please check the effective permissions.','pressbackup'), $pressbackup->Path->Dir['LOGTMP']);
			$disable_backup_files = true;
		} else {
			@chmod($pressbackup->Path->Dir['LOGTMP'], 0777);
		}

		$pressbackup->View->set('error', $error);
		$pressbackup->View->set('disable_backup_files', $disable_backup_files);
	}


	/**
	 * Show the form to select the service:
	 * S3, Pressbackup Pro, localhost, Dropbox, etc
	 */
	function config_step1() {

		global $pressbackup;
		$pressbackup->View->layout('principal');

		$service = false;
		if ($pressbackup->Session->check('error_msg')) {

			//save selected credentials && set error msg
			$service = $pressbackup->Session->read('service');
			$pressbackup->Msg->error($pressbackup->Session->read('error_msg'));

			$pressbackup->Session->delete('error');
			$pressbackup->Session->delete('service');
			$pressbackup->Session->delete('error_msg');
		}

		$pressbackup->View->set('service', $service);
		$pressbackup->View->set('settings', get_option('pressbackup_preferences'));
	}

	/**
	 * Store the credentials if they are correct
	 * or save the error and return to form page
	 */
	function config_step1_save() {

		global $pressbackup;
		$preferences = get_option('pressbackup_preferences');

		//Exit if wrong parameters
		if (!isset($_POST['data']) || !$this->check_fields_step1($_POST['data'])) {
			$pressbackup->Session->write('error', true);
			$pressbackup->Session->write('service', $_POST['data']['service']);
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_step1'));
		}

		if ($_POST['data']['service'] == 's3') {

			$PS3 = $_POST['data']['preferences']['S3'];

			$pressbackup->import('S3.php');
			$s3 = new PressbackupS3($PS3['accessKey'], $PS3['secretKey']);

			//check if bucket exists
			$buckets = @$s3->listBuckets();
			$the_bucket = $pressbackup->Config->read('S3.bucketname') . '-' . md5(uniqid(rand(), true));
			$create_bucket = true;
			for ($i = 0; $i < count($buckets); $i++) {
				if (strpos($buckets[$i], $pressbackup->Config->read('S3.bucketname')) !== false) {
					$the_bucket = $buckets[$i];
					$create_bucket = false;
					break;
				}
			}

			//create bucket if it not exist
			$region = ($PS3['region'] == 'EU') ? 'EU' : false;
			if ($create_bucket && !$s3->putBucket($the_bucket, PressbackupS3::ACL_PRIVATE, $region)) {
				$pressbackup->Session->write('error_msg', __("Unable to create a bucket on S3: Service temporarily unavailable. Please try it again later.",'pressbackup'));
				return false;
			}

			//values to save
			$preferences['pressbackup']['s3']['bucket_name'] = $the_bucket;
			$preferences['pressbackup']['s3']['region'] = $region;
			$preferences['pressbackup']['s3']['credential'] = base64_encode($PS3['accessKey'] . '|AllYouNeedIsLove|' . $PS3['secretKey']);
			$preferences['pressbackup']['pressbackuppro']['credential'] = '';
			$preferences['pressbackup']['dropbox']['credential'] = '';
			$preferences['pressbackup']['local']['path'] = '';

		} elseif ($_POST['data']['service'] == 'pro'){

			$PPRO = $_POST['data']['preferences']['pressbackuppro'];

			$preferences['pressbackup']['pressbackuppro']['credential'] = base64_encode($PPRO['user'] . '|AllYouNeedIsLove|' . $PPRO['key']);
			$preferences['pressbackup']['s3']['credential'] = '';
			$preferences['pressbackup']['dropbox']['credential'] = '';
			$preferences['pressbackup']['local']['path'] = '';

		} elseif ($_POST['data']['service'] == 'local') {

			$PLOCAL = $_POST['data']['preferences']['local'];

			$preferences['pressbackup']['s3']['credential'] = '';
			$preferences['pressbackup']['pressbackuppro']['credential'] = '';
			$preferences['pressbackup']['dropbox']['credential'] = '';
			$preferences['pressbackup']['local']['path'] = $PLOCAL['path'];

		} else {

			$preferences['pressbackup']['s3']['credential'] = '';
			$preferences['pressbackup']['pressbackuppro']['credential'] = '';
			$preferences['pressbackup']['dropbox']['credential'] = '';
			$preferences['pressbackup']['local']['path'] = '';
		}

		update_option('pressbackup_preferences', $preferences);


		if ($_POST['data']['service'] == 'dbox') {
			$pressbackup->import('dropbox.php');
			$dbox = new PressbackupDropbox();
			$dbox->auth(array('menu_type' => 'settings', 'controller' => 'settings', 'function' => 'dropbox_authorize'));
		} else {
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_step2'));
		}
	}


	/**
	 * Show the form to enter prefered settings
	 * as backup type, schedule time, etc
	 */
	function config_step2() {

		global $pressbackup;
		$pressbackup->View->layout('principal');

		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		$preferences = get_option('pressbackup_preferences');

		//check if there are errors to show
		if ($pressbackup->Session->check('error')) {
			$pressbackup->Msg->error($pressbackup->Session->read('error_msg'));
			$pressbackup->Session->delete('error');
			$pressbackup->Session->delete('error_msg');
		}

		$pressbackup->View->set('settings', $preferences);
		$pressbackup->View->set('service', $misc->current_service());
	}

	/**
	 * Store the settings if they are correct
	 * or save the error and return to form page
	 */
	function config_step2_save() {

		global $pressbackup;
		$preferences = get_option('pressbackup_preferences');

		$pressbackup->import('backup_functions.php');
		$pb = new pressbackup_backup();

		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		//exit if no data or invalid
		if (!isset($_POST['data']) || !$this->check_fields_step2($_POST['data']['preferences'])) {
			$pressbackup->Session->write('error', true);
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_step2'));
		}

		//Check if davanced options were activated
		$advanced_activated = $_POST['data']['preferences']['backup_advanced']['activated'];

		//saving number of copies to store
		if (isset($_POST['data']['preferences']['copies'])) {
			$preferences['pressbackup']['backup']['copies'] = $_POST['data']['preferences']['copies'];
		} else {
			$preferences['pressbackup']['backup']['copies'] = 5;
		}

		// saving backup schedule time
		if ($advanced_activated) {
			$scheduled = $pb->min_time($_POST['data']['preferences']['backup_advanced']);
			$preferences['pressbackup']['backup']['time'] = $scheduled['time'];
		}else{
			$preferences['pressbackup']['backup']['time'] = $_POST['data']['preferences']['time'];
		}

		//setting backup type
		if($advanced_activated){
			$curdate = strtotime(date('Y-m-d H:i:s'));
			$post = $_POST['data']['preferences']['backup_advanced'];

			$preferences['pressbackup']['backup_advanced']['activated'] = true;
			$preferences['pressbackup']['backup_advanced']['db']= array('time' => $post['db']['time'], 'last_date' => $curdate);
			$preferences['pressbackup']['backup_advanced']['plugins']= array('time' => $post['plugins']['time'], 'last_date' => $curdate);
			$preferences['pressbackup']['backup_advanced']['themes']= array('time' => $post['themes']['time'], 'last_date' => $curdate);
			$preferences['pressbackup']['backup_advanced']['uploads']= array('time' => $post['uploads']['time'], 'last_date' => $curdate);

			$preferences['pressbackup']['backup']['type'] = $pb->activated_types($preferences['pressbackup']['backup_advanced']);
		}else{
			$preferences['pressbackup']['backup_advanced']['activated'] = false;
			$preferences['pressbackup']['backup']['type'] = join(',', $_POST['data']['preferences']['type']);
		}

		$preferences['pressbackup']['configured'] = 1;

		update_option('pressbackup_preferences', $preferences);

		if ($misc->current_service()) {
			$pb->add_schedule();
		}

		$pressbackup->redirect(array('menu_type' => 'tools', 'controller' => 'principal', 'function' => 'dashboard'));
	}


	/**
	 * Show the form to modify the service:
	 * S3, Pressbackup Pro, localhost, Dropbox, etc
	 */
	function config_credentials() {

		global $pressbackup;
		$pressbackup->View->layout('principal');

		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		$cs = $misc->current_service();
		$service = $cs['id'];

		if ($pressbackup->Session->check('error')) {

			$pressbackup->Msg->error($pressbackup->Session->read('error_msg'));
			$service = $pressbackup->Session->read('service');

			$pressbackup->Session->delete('error');
			$pressbackup->Session->delete('service');
			$pressbackup->Session->delete('error_msg');
		}

		$pressbackup->View->set('service', $service);
		$pressbackup->View->set('services', $misc->service());
		$pressbackup->View->set('settings', get_option('pressbackup_preferences'));

	}

	/**
	 * Store the modified credentials if they are correct
	 * or save the error and return to form page
	 */
	function config_credentials_save() {

		global $pressbackup;
		$pressbackup->import('backup_functions.php');
		$pressbackup->import('misc.php');

		if (!isset($_POST['data']) || !$this->check_fields_step1($_POST['data'])) {
			$pressbackup->Session->write('error', true);
			$pressbackup->Session->write('service', $_POST['data']['service']);
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_credentials'));
		}

		$preferences = get_option('pressbackup_preferences');

		if ($_POST['data']['service'] == 's3') {

			$PS3 = $_POST['data']['preferences']['S3'];

			$pressbackup->import('S3.php');
			$s3 = new PressbackupS3($PS3['accessKey'], $PS3['secretKey']);

			//check if bucket exists
			$buckets = @$s3->listBuckets();
			$the_bucket = $pressbackup->Config->read('S3.bucketname') . '-' . md5(uniqid(rand(), true));
			$create_bucket = true;
			for ($i = 0; $i < count($buckets); $i++) {
				if (strpos($buckets[$i], $pressbackup->Config->read('S3.bucketname')) !== false) {
					$the_bucket = $buckets[$i];
					$create_bucket = false;
					break;
				}
			}

			//create bucket if it not exist
			$region = ($PS3['region'] == 'EU') ? 'EU' : false;
			if ($create_bucket && !$s3->putBucket($the_bucket, PressbackupS3::ACL_PRIVATE, $region)) {
				$pressbackup->Session->write('error_msg', __("Unable to create a bucket on S3: Service temporarily unavailable. Please try it again later.",'pressbackup'));
				return false;
			}

			//values to save
			$preferences['pressbackup']['s3']['bucket_name'] = $the_bucket;
			$preferences['pressbackup']['s3']['region'] = $region;
			$preferences['pressbackup']['s3']['credential'] = base64_encode($PS3['accessKey'] . '|AllYouNeedIsLove|' . $PS3['secretKey']);
			$preferences['pressbackup']['pressbackuppro']['credential'] = '';
			$preferences['pressbackup']['dropbox']['credential'] = '';
			$preferences['pressbackup']['local']['path'] = '';

		} elseif ($_POST['data']['service'] == 'pro') {

			$PPRO = $_POST['data']['preferences']['pressbackuppro'];
			$preferences['pressbackup']['s3']['credential'] = '';
			$preferences['pressbackup']['pressbackuppro']['credential'] = base64_encode($PPRO['user'] . '|AllYouNeedIsLove|' . $PPRO['key']);
			$preferences['pressbackup']['dropbox']['credential'] = '';
			$preferences['pressbackup']['local']['path'] = '';

		} elseif ($_POST['data']['service'] == 'local') {

			$PLOCAL = $_POST['data']['preferences']['local'];
			$preferences['pressbackup']['s3']['credential'] = '';
			$preferences['pressbackup']['pressbackuppro']['credential'] = '';
			$preferences['pressbackup']['dropbox']['credential'] = '';
			$preferences['pressbackup']['local']['path'] = $PLOCAL['path'];

		} elseif ( in_array($_POST['data']['service'], array('no', 'dbox'))) {

			$preferences['pressbackup']['s3']['credential'] = '';
			$preferences['pressbackup']['pressbackuppro']['credential'] = '';
			$preferences['pressbackup']['dropbox']['credential'] = '';
			$preferences['pressbackup']['local']['path'] = '';
			$preferences['pressbackup']['backup_advanced']['activated'] = false;
		}

		update_option('pressbackup_preferences', $preferences);

		//remove schedule if sleected none, o add it id selected other
		$misc = new pressbackup_misc();
		$pb = new pressbackup_backup();
		if ($misc->current_service()) { $pb->add_schedule(); } else { $pb->remove_schedule(); }

		//redirect to dropbox if selected or finish flow
		if ($_POST['data']['service'] == 'dbox') {
			$pressbackup->import('dropbox.php');
			$dbox = new PressbackupDropbox();
			$dbox->auth(array('menu_type' => 'settings', 'controller' => 'settings', 'function' => 'dropbox_authorize'));
		} else {
			$pressbackup->Session->write('general_msg', __('Service Saved!','pressbackup'));
			$pressbackup->redirect(array('menu_type' => 'tools', 'controller' => 'principal', 'function' => 'dashboard'));
		}
	}


	/**
	 * Show the form to modify prefered settings
	 * as backup type, schedule time, etc
	 */
	function config_settings() {

		global $pressbackup;
		$pressbackup->View->layout('principal');
		$preferences = get_option('pressbackup_preferences');

		if ($pressbackup->Session->check('error')) {
			$pressbackup->Msg->error($pressbackup->Session->read('error_msg'));
			$pressbackup->Session->delete('error');
			$pressbackup->Session->delete('error_msg');
		}

		//Check for the current service and its credentials, if they are available
		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		$pressbackup->View->set('settings', get_option('pressbackup_preferences'));
		$pressbackup->View->set('service', $misc->current_service());
	}

	/**
	 * Store the modified settings if they are correct
	 * or save the error and return to form page
	 */
	function config_settings_save() {

		global $pressbackup;
		$preferences = get_option('pressbackup_preferences');

		$pressbackup->import('backup_functions.php');
		$pb = new pressbackup_backup();


		//exit if no data or invalid
		if (!isset($_POST['data']) || !$this->check_fields_step2($_POST['data']['preferences'])) {
			$pressbackup->Session->write('error', true);
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_settings'));
		}

		//Check if davanced options were activated
		$advanced_activated = $_POST['data']['preferences']['backup_advanced']['activated'];

		//saving number of copies to store
		if (isset($_POST['data']['preferences']['copies'])) {
			$preferences['pressbackup']['backup']['copies'] = $_POST['data']['preferences']['copies'];
		} else {
			$preferences['pressbackup']['backup']['copies'] = 5;
		}

		// saving backup schedule time
		if ($advanced_activated) {
			$scheduled = $pb->min_time($_POST['data']['preferences']['backup_advanced']);
			$preferences['pressbackup']['backup']['time'] = $scheduled['time'];
			$pb->add_schedule('+'.$preferences['pressbackup']['backup']['time'].' hours');
		}
		elseif ( isset ($_POST['data']['preferences']['time']) ) {
			$preferences['pressbackup']['backup']['time'] = $_POST['data']['preferences']['time'];
			$pb->add_schedule('+'.$preferences['pressbackup']['backup']['time'].' hours');
		}
		else {
			$preferences['pressbackup']['backup']['time'] = 0;
			$pb->remove_schedule();
		}

		//setting backup type
		if($advanced_activated){
			$curdate = strtotime(date('Y-m-d H:i:s'));
			$post = $_POST['data']['preferences']['backup_advanced'];

			$preferences['pressbackup']['backup_advanced']['activated'] = true;
			$preferences['pressbackup']['backup_advanced']['db']= array('time' => $post['db']['time'], 'last_date' => $curdate);
			$preferences['pressbackup']['backup_advanced']['plugins']= array('time' => $post['plugins']['time'], 'last_date' => $curdate);
			$preferences['pressbackup']['backup_advanced']['themes']= array('time' => $post['themes']['time'], 'last_date' => $curdate);
			$preferences['pressbackup']['backup_advanced']['uploads']= array('time' => $post['uploads']['time'], 'last_date' => $curdate);

			$preferences['pressbackup']['backup']['type'] = $pb->activated_types($preferences['pressbackup']['backup_advanced']);
		}else{
			$preferences['pressbackup']['backup_advanced']['activated'] = false;
			$preferences['pressbackup']['backup']['type'] = join(',', $_POST['data']['preferences']['type']);
		}

		update_option('pressbackup_preferences', $preferences);

		$pressbackup->Session->write('general_msg', __('Preferences Saved!','pressbackup'));
		$pressbackup->redirect(array('menu_type' => 'tools', 'controller' => 'principal', 'function' => 'dashboard'));

	}


	/**
	 * Show the form to modify prefered conpatibility settings
	 */
	function config_compatibility() {

		global $pressbackup;
		$pressbackup->View->layout('principal');
		$preferences = get_option('pressbackup_preferences');

		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		$binaries = false;
		if ($misc->checkShell('php')) {
			$binaries = true;
		}

		$pressbackup->View->set('binaries', $binaries);
		$pressbackup->View->set('settings', $preferences);
	}

	/**
	 * Store the modified settings if they are correct
	 * or save the error and return to form page
	 */
	function config_compatibility_save() {

		global $pressbackup;

		$preferences = get_option('pressbackup_preferences');

		if (isset($_POST['data']['compatibility']['background'])) {
			$preferences['pressbackup']['compatibility']['background'] = $_POST['data']['compatibility']['background'];
		}

		if (isset($_POST['data']['compatibility']['zip'])) {
			$preferences['pressbackup']['compatibility']['zip'] = $_POST['data']['compatibility']['zip'];
		}

		update_option('pressbackup_preferences', $preferences);

		$pressbackup->Session->write('general_msg', __('Preferences Saved!','pressbackup'));
		$pressbackup->redirect(array('menu_type' => 'tools', 'controller' => 'principal', 'function' => 'dashboard'));
	}


	/**
	 * Store the settings if they are correct
	 * or save the error and return to form page
	 *
	 * @token_s string: token generated on revios step
	 */
	function dropbox_authorize($token_s) {
		global $pressbackup;

		$pressbackup->import('dropbox.php');
		$dbox = new PressbackupDropbox();

		$token = $_GET['oauth_token'];
		$tk = $dbox->get_access_token($token, $token_s);

		$preferences = get_option('pressbackup_preferences');

		$preferences['pressbackup']['dropbox']['credential'] = base64_encode($tk['oauth_token'] . '|AllYouNeedIsLove|' . $tk['oauth_token_secret']);
		update_option('pressbackup_preferences', $preferences);

		if($preferences['pressbackup']['configured']){
			$pressbackup->Session->write('general_msg', __('Service Saved!','pressbackup'));
			$pressbackup->redirect(array('menu_type' => 'tools', 'controller' => 'principal', 'function' => 'dashboard'));
		} else {
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_step2'));
		}
	}

	/**
	 * Check if credentials are valid
	 * for S3 or Pressbackup PRO account
	 *
	 * @args array: credentials sent via post
	 */
	function check_fields_step1($args) {

		global $pressbackup;

		if ($args['service'] == 's3') {
			$PS3 = $args['preferences']['S3'];

			//check empty data
			if (!isset($PS3['accessKey']) || empty($PS3['accessKey']) || !isset($PS3['secretKey']) || empty($PS3['secretKey'])) {
				$pressbackup->Session->write('error_msg', __('Empty Fields, please ty it again','pressbackup'));
				return false;
			}

			$pressbackup->import('S3.php');
			$s3 = new PressbackupS3($PS3['accessKey'], $PS3['secretKey']);

			//check if credentials are right
			if (($buckets = @$s3->listBuckets()) === false) {
				$pressbackup->Session->write('error_msg', __('Incorrect S3 access or secret Keys','pressbackup'));
				return false;
			}

			return true;
		} elseif ($args['service'] == 'pro') {
			$pressbackup->import('Pro.php');

			$PPRO = $args['preferences']['pressbackuppro'];

			if (!isset($PPRO['key']) || empty($PPRO['key']) || !isset($PPRO['user']) || empty($PPRO['user'])) {
				$pressbackup->Session->write('error_msg', __('Empty Fields, please ty it again','pressbackup'));
				return false;
			}

			$pro = new pressbackup($PPRO['user'], $PPRO['key']);
			if ($pro->auth() === false || $pro->response_code == 401) {
				$pressbackup->Session->write('error_msg', __('Incorrect PressBackup Pro User/AuthKey','pressbackup'));
				return false;
			}

			elseif (!$pro->check()) {
				$pressbackup->Session->write('error_msg', __("This blog is not registered on your Pressbackup Pro account! Register it and try again",'pressbackup'));
				return false;
			}

			return true;
		} elseif ($args['service'] == 'local') {

			$PLOCAL = $args['preferences']['local'];

			$pressbackup->import('local.php');
			$local = new PressbackupLocal();

			if (!$local->check_exists($PLOCAL['path'])) {
				$pressbackup->Session->write('error_msg', __("The specified folder does not exist.",'pressbackup'));
				return false;
			}
			if (!$local->check_permissions($PLOCAL['path'])) {
				$pressbackup->Session->write('error_msg', __("No write permissions for the selected folder",'pressbackup'));
				return false;
			}
			return true;
		}

		return true;
	}

	/**
	 * Check if fields for Settings are valid.
	 * The files are type of backup, schedule time, etc
	 *
	 * @args array: all fields sent via post
	 */
	function check_fields_step2($args) {

		global $pressbackup;
		$pass = true;

		if(!isset($args['backup_advanced']['activated']) ){

			if (isset($args['time']) && !preg_match("/^[[:digit:]]+$/", $args['time'])) {
				$pressbackup->Session->write('error_msg', __('Specify the scheduled backup time range','pressbackup'));
				return false;
			}

			if (isset($args['copies']) && !preg_match("/^[[:digit:]]+$/", $args['copies'])) {
				$pressbackup->Session->write('error_msg', __('Specify the number of backups to be stored','pressbackup'));
				return false;
			}

			if (!isset($args['type']) || empty($args['type'])) {
				$pressbackup->Session->write('error_msg', __('Specify what you would like to backup','pressbackup'));
				return false;
			}

			for ($i = 0; $i < count($args['type']); $i++) {
				if (!preg_match("/^[[:digit:]]+$/", $args['type'][$i])) {
					$pressbackup->Session->write('error_msg', __('Empty/wrong Fields, please ty it again','pressbackup'));
					return false;
				}
			}

		}else {
			unset($args['backup_advanced']['activated']);
			$at_least_one = false;

			//check that all times received they are digit, and that at least one is activated,
			foreach($args['backup_advanced'] as $type=>$data){
				if (isset($data['time']) && !preg_match("/^[[:digit:]]+$/", $data['time'])) {
					$pressbackup->Session->write('error_msg', __('Specify the scheduled backup time range','pressbackup'));
					return false;
				}
				if($data['time'] != 0){
					$at_least_one = true;
				}
			}

			if(!$at_least_one){
				$pressbackup->Session->write('error_msg', __('Specify what you would like to backup','pressbackup'));
				return false;
			}
		}
		return true;
	}

}
?>
